#!/usr/bin/perl
#
# (C) 2001-2002
#	Domsodi Gergely	<doome@uhulinux.hu>
#	Pozsar Balazs	<pozsy@uhulinux.hu>
#
# This file is originally part of the UHU Linux distribution.
# Distributable under GPL v2.
#

# ----- Pragmatic modules -----------------------------------------------------
use strict;
use warnings;

# -----------------------------------------------------------------------------
package UHU::gtk2;

use Gtk2;
use Gtk2::GladeXML;

use UHU::common;
use Time::HiRes qw(gettimeofday);

# ----- Exporter --------------------------------------------------------------
require Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
set_combo_list set_combo_hash get_combo_key
insert_node
w gps ifs
create_columns get_row_data set_row_data
plug load_ui load_ui_into_container load_ui_into_plug
load_image setup_images image
shrink _shrink
set_cursor
set_if_not
Xres
screenshot
screenshot_keysnooper
focus popup msg msgwrap yesno noyes okcancel cancelok
dir_select
gtk_iterate
html2view
);

# -----------------------------------------------------------------------------
my %combodata;
my %images;
my $images;

sub set_combo_list($$$) {
	my ($g, $combo, $list) = @_;

	debug "set_combo_list";
	debug "-> $_" foreach (@$list);

	w($g, $combo)->set_popdown_strings(@$list);
}

sub set_combo_hash($$$) {
	my ($g, $combo, $hash) = @_;

	debug "set_combo_hash";

	$combodata{$g}{$combo} = $hash;

	set_combo_list $g, $combo, [sort keys %$hash];

	#	set_combo_list $g, $combo, [sort(map {$hash->{$_}->{text}} keys(%$hash))];
}

sub get_combo_key($$) {
	my ($g, $combo) = @_;

	$combodata{$g}{$combo}{ w($g, $combo)->entry->get_text };
}

sub gtk_iterate() {
	Gtk2->main_iteration while Gtk2->events_pending;
}

sub create_columns($$$) {
	my ($treeview, $tl, $data) = @_;
	my @tmp;
	my $store;
	my $renderer;
	foreach (sort keys %{$data}) {
		if ( $data->{$_}->{type} ) {
			if ( $data->{$_}->{type}  eq 'bool' or 
					$data->{$_}->{type} eq 'radio') {
				push @tmp,"Glib::Boolean";
			} elsif ( $data->{$_}->{type} eq 'int') {
				push @tmp,"Glib::Int";
			} elsif ( $data->{$_}->{type} eq 'pixmap') {
				push @tmp,"Gtk2::Gdk::Pixbuf";
			} else {
				push @tmp,"Glib::String";
			}
		} else {
			push @tmp,"Glib::String";
		}
	}
	$store = ($tl eq 'tree') ? 
				new Gtk2::TreeStore(@tmp) :
				new Gtk2::ListStore(@tmp);
	$treeview->set_model($store);
	foreach (sort keys %{$data}) {
		if ( $data->{$_}->{type} and !$data->{$_}->{hidden}) {
			if ( $data->{$_}->{type} eq 'bool' or 
					$data->{$_}->{type} eq 'radio') {
				$renderer = new Gtk2::CellRendererToggle;
				$treeview->insert_column_with_attributes (-1, $data->{$_}->{title}, 
					$renderer,
					active => $_,
					%{$data->{$_}->{data}}); 
					$renderer->set_radio(1)
						if ($data->{$_}->{type} eq 'radio');
				$renderer->signal_connect(toggled => \&main::item_toggled,$store);
			} elsif ( $data->{$_}->{type} eq 'pixmap' ) {
				$renderer = new Gtk2::CellRendererPixbuf;
				$treeview->insert_column_with_attributes (-1, $data->{$_}->{title}, 
					$renderer,
					pixbuf => $_,
					%{$data->{$_}->{data}}); 
			} else {
				$renderer = new Gtk2::CellRendererText;
				$treeview->insert_column_with_attributes (-1, $data->{$_}->{title}, 
						$renderer, 
						text => $_,
						%{$data->{$_}->{data}}); 
			}
		} elsif (!$data->{$_}->{hidden}) {
				$renderer = new Gtk2::CellRendererText;
				$treeview->insert_column_with_attributes (-1, $data->{$_}->{title}, 
						$renderer, 
						text => $_,
						%{$data->{$_}->{data}}); 
		}
	}
	$treeview->set_rules_hint(1);
	return $store;
}

# csak egy rovidites a $treemodel->get_path helyett
sub gps($$) {
	my ($treemodel, $iterator) = @_;

	($treemodel->get_path($iterator))->to_string;
}

# szinten csak egy rovidites...
sub ifs($$) {
	my ($treemodel,$pathstring) = @_;

	$treemodel->get_iter_from_string($pathstring);
}

sub get_row_data($$) {
	my ($treemodel,$iterator) = @_;

	$treemodel->{gps($treemodel, $iterator)};
}

sub set_row_data($$$) {
	my ($treemodel,$iterator,$data) = @_;

	$treemodel->{gps($treemodel,$iterator)} = $data;
}

sub plug($) {
	my ($wid) = @_;

	die "Nincs hova csatlakozni! (nincs WID)" unless $wid;

	my $plug = new Gtk2::Plug $wid;
	$plug->signal_connect('destroy',      sub {exit});
	$plug->signal_connect('delete_event', sub {exit});

	$plug->show;

	$plug;
}

sub w($$) {
	my ($root, $widget_name) = @_;

	$root->get_widget($widget_name);
}

sub load_ui($) {
	my ($uifile) = @_;

	my $g = new Gtk2::GladeXML($uifile);
	$g->signal_autoconnect_from_package('main');

	$g;
}

sub load_ui_into_container($$$) {
	my ($uifile, $sourcename, $target) = @_;

	my $g = load_ui($uifile);

	my $source = w($g, $sourcename);

	$source->unrealize;
	$source->hide;
	$source->reparent($target);
	$source->show;
	$source->realize;

	$g;
}

sub load_ui_into_plug($$$) {
	my ($uifile, $sourcename, $XID) = @_;

	load_ui_into_container $uifile, $sourcename, plug($XID);
}

sub load_image($) {
	my ($name) = @_;

	debug "loading image: [$name]";

	Gtk2::Gdk::Pixbuf->new_from_file($name);
}

sub setup_images($) {
	my ($hash) = @_;

	$images = $hash;
}

sub image($) {
	my ($id) = @_;

	$images{$id} ||= load_image $images->{$id};
}

sub set_cursor(;$$) {
	my ($widget, $cursor) = @_;

	(($widget and $widget->window) or Gtk2::Gdk->get_default_root_window)->set_cursor(Gtk2::Gdk::Cursor->new($cursor || 'left-ptr'));
	gtk_iterate;
}

sub user_res($) {
	my ($window) = @_;
	$window->set_resizable(1);
	0;
}

sub _shrink($) {
	my ($window) = @_;
	$window->set_resizable(0);
	Glib::Idle->add(\&user_res,$window);
}

sub shrink(;$) {
	my ($window) = @_;
	if ($ENV{JUHHU_PID}){
		debug 'sending SIGWINCH to '.$ENV{JUHHU_PID};
		kill 28, $ENV{JUHHU_PID};
	} elsif ($window) {
		_shrink($window);
	} 
	0;
}

# beallitja az entry tartalmat, ha nem az hogy ne kuldjon megint signalt
sub set_if_not($$) { 
	my ($entry, $text) = @_;
	$text ||= '';
	$entry->set_text($text) if ($entry->get_text ne $text);	
}

# visszaadja az aktualis kepernyo-felbontast
sub Xres {
	my $screen = Gtk2::Gdk::Screen->get_default;
	(Gtk2::Gdk::Screen::get_width($screen), 
		Gtk2::Gdk::Screen::get_height($screen));
}

sub screenshot($$) {
	my ($filename, $drawable) = @_;
	my ($width, $height) = $drawable->get_size;
	my $pixbuf = Gtk2::Gdk::Pixbuf->get_from_drawable($drawable, undef, 0, 0, 0, 0, $width, $height);
	my $dir = $ENV{UHU_LOGDIR} ? $ENV{UHU_LOGDIR}.'/screenshots/' : 
								 $ENV{HOME}.'/';
	msg(_("Screenshot saved to: %s",$dir.$filename));
	$pixbuf->save($dir.$filename,'png');
}

sub screenshot_keysnooper {
	my ($window, $key, $module) = @_;
	if ($key->type eq 'key-press' and $key->keyval eq 65377) { #printscreen
		my ($seconds,$microsec) = gettimeofday;
	    my ($sec,$min,$hour,$mday,$mon,$year,undef,undef,undef) =
			localtime($seconds);
		screenshot($year+1900 . pad($mon) . pad($mday) . '-' .
				     pad($hour) . pad($min) . pad($sec) .'-' .
				$microsec.'-'.$module.'.png', $window->window);
		return 1;
	}
	0;
}

#sub __popup(%) {
sub popup(%) {
#	my $handlerpid = shift;
	my %prop = @_;
	debug "Popup with message:".$prop{msg};

	my $popup = new Gtk2::Window 'toplevel';
	$popup->set_position('center');
	$popup->set_decorated(0);
	$popup->set_modal(1);
	$popup->signal_connect('delete_event', sub {1});

	my $frame1 = new Gtk2::Frame;
	$frame1->set_shadow_type('out');
	$popup->add($frame1);

	my $frame2 = new Gtk2::Frame;
	$frame2->set_shadow_type('in');
	$frame2->set_border_width(3);
	$frame1->add($frame2);

	my $vbox = new Gtk2::VBox(0, 10);
	$vbox->set_border_width(10);
	$frame2->add($vbox);

	my $label = new Gtk2::Label;
	$label->set_markup($prop{msg});
	$label->set_line_wrap(1) if $prop{wrap};
	if ($prop{scrolled}) {
		my $scrolledwindow = new Gtk2::ScrolledWindow;
		$vbox->pack_start_defaults($scrolledwindow);
		$scrolledwindow->add_with_viewport($label);
		$scrolledwindow->set_policy('automatic','automatic');
		$scrolledwindow->set_size_request(400,300);
	} else {
		$vbox->pack_start_defaults($label);
	}
	$vbox->add(new Gtk2::HSeparator);

	my $hbuttonbox = new Gtk2::HButtonBox;
	$vbox->pack_start($hbuttonbox, 0, 0, 0);

	$prop{buttons} ||= ['stock:gtk-ok', undef];
	while (@{ $prop{buttons} }) {
		my $label   = shift @{ $prop{buttons} };
		my $handler = shift @{ $prop{buttons} };
		my @argv = ();
		if (ref($handler) eq 'ARRAY') {
			($handler, @argv) = @$handler;
		}

		my $button;
		if ($label =~ m/^stock:(.+)/) {
			my $label2 = $1;
			$button = Gtk2::Button->new_from_stock($label2);
		} else {
			$button = Gtk2::Button->new_with_label($label);
		}
		$button->signal_connect(
			'clicked',
			sub {
				$popup->hide;
				$popup->destroy;
#				&main::ipfc($handlerpid, $handler, @argv) if $handler;
				gtk_iterate;
				&$handler(@argv) if $handler;
				Gtk2->main_quit if $prop{blocking};
				0;
			}
		);
		$hbuttonbox->add($button);
		$button->grab_focus if ($prop{default} or $label) eq $label;
	}
	set_cursor $popup;
	$popup->show_all;
	gtk_iterate;
	Gtk2->main if $prop{blocking};
	$popup;
}

#sub _popup(@) {
#	__popup($$,@_);
#}

#sub popup(@) {
#	&main::ipfc(($ENV{JUHHU_PID} or $$),'__popup',$$,@_);
#}

# wrapper a bugos grab_focus hívás köré
sub focus($) {
	my ($widget) = @_;
	
	Glib::Idle->add(sub{
		$widget->grab_focus;
		0;
	});
}

sub msg($;$) {
	my ($msg, $handler) = @_;

	popup(msg => $msg, buttons => [ 'stock:gtk-ok' => ($handler or undef) ]);
}

sub msgwrap($;$) {
	my ($msg, $handler) = @_;

	popup(msg => $msg, wrap => 1, buttons => [ 'stock:gtk-ok' => ($handler or undef) ]);
}

sub yesno($$;$) {
	my ($msg, $yes, $no) = @_;

	popup(msg => $msg, buttons => [ 'stock:gtk-yes' => $yes, 'stock:gtk-no' => $no ]);
}

sub noyes($$;$) {
	my ($msg, $no, $yes) = @_;

	popup(msg => $msg, buttons => [ 'stock:gtk-no' => $no, 'stock:gtk-yes' => $yes ]);
}

sub okcancel($$;$) {
	my ($msg, $ok, $cancel) = @_;

	popup(msg => $msg, buttons => [ 'stock:gtk-ok' => $ok, 'stock:gtk-cancel' => $cancel ]);
}

sub cancelok($$;$) {
	my ($msg, $cancel, $ok) = @_;

	popup(msg => $msg, buttons => [ 'stock:gtk-cancel' => $cancel, 'stock:gtk-ok' => $ok ]);
}

sub dir_select($$) {
	my ($title, $callback) = @_;

	my $selector = new Gtk2::FileSelection($title);
	$selector->set_modal(1);
	$selector->hide_fileop_buttons;

	#	$selector->file_list->set_sensitive(0);
	#	$selector->file_list->signal_handlers_destroy;
	$selector->file_list->parent->hide;

	#	$selector->selection_entry->set_sensitive(0);
	#	$selector->selection_entry->hide;
	$selector->selection_text->hide;
	$selector->ok_button->signal_connect(
		'clicked',
		sub {
			my $dir = $selector->get_filename;
			$dir =~ s|/+|/|g;
			if (-d $dir) {
				&$callback($dir);
				$selector->destroy;
			} else {
				popup(msg => 'Nincs ilyen könyvtár!');
			}
		}
	);
	$selector->cancel_button->signal_connect(
		'clicked',
		sub {
			$selector->destroy;
		}
	);
	$selector->show;
}

sub _html2view_request_url {
	my ($document, $url, $stream) = @_;

	$url =~ s%file://%%;

	$stream->set_mime_type("image/jpeg") if ($url =~ /\.jpe?g$/);
	$stream->set_mime_type("image/png")  if ($url =~ /\.png$/);
	open F, $url;
	binmode F;
	while (read F, my $data, 1024) {
		$stream->write($data);
	}
	close F;
}

sub _html2view($) {
	my ($filename) = @_;

	my $scrolledwindow = new Gtk2::ScrolledWindow;
	my $view = new Gtk2::Html2::View;
	my $document = new Gtk2::Html2::Document;

	$document->signal_connect(request_url => \&_html2view_request_url);
#	$document->signal_connect(link_clicked => \&_html2view_link_clicked);

	$view->set_document($document);

	$document->open_stream("text/html");
	$document->write_stream(scalar cat $filename);

	$scrolledwindow->add($view);

	$scrolledwindow->show_all;
	$scrolledwindow;
}

sub html2view($) {
	my ($name) = @_;

	require Gtk2::Html2;

	$name =~ s%\.html$%%;

	my $lang = $ENV{LC_ALL} || $ENV{LC_MESSAGES} || $ENV{LANG};
	$lang =~ s%@.*%%;
	$lang =~ s%\..*%%;
	return _html2view($name.".".$lang.".html") if (-f $name.".".$lang.".html");
	$lang =~ s%_.*%%;
	return _html2view($name.".".$lang.".html") if (-f $name.".".$lang.".html");
	_html2view($name.".html");
}

# ----- Initialization --------------------------------------------------------
init Gtk2;

# ----- Hack -- DO NOT REMOVE -------------------------------------------------
Glib::Timeout->add(100, sub { 1 });

# ----- Beatiful perl :) ------------------------------------------------------
"Megy a csiga hazafele.";
